==> 0.0.2.tar.gz: lib/controls/combination_pool.dart <==
import 'package:flutter/material.dart';
import 'package:select_form_field/select_form_field.dart';

import 'package:bizdoc/services/cube.dart';
import 'package:bizdoc/services/datasource.dart';
import 'package:bizdoc/services/session.dart';
import 'package:bizdoc/models/profile.dart';

typedef AxesArgs = void Function(Map<String, dynamic> axes);

class CombinationPool extends StatefulWidget {
  final String? labelText;
  final String? initialValue;
  final AxesArgs onChanged;
  final String? cube;
  CombinationPool(
      {this.cube, required this.onChanged, this.labelText, this.initialValue});
  @override
  _CombinationPoolState createState() => _CombinationPoolState();
}

class _CombinationPoolState extends State<CombinationPool> {
  final Map<String, dynamic> _values = {};
  final Map<String, Map<String, dynamic>> _datasources = {};

  late CubeInfo cubeInfo;
  late Iterable<CubeAxis> axes;

  @override
  void initState() {
    cubeInfo = session.profile.cubes.firstWhere(
        (element) => widget.cube == null || element.name == widget.cube);
    axes = cubeInfo.axes.where((element) => element.combination);
    super.initState();
  }

  _initialize() async {
    final values =
        widget.initialValue != null ? widget.initialValue!.split('.') : null;
    for (var i = 0; i < axes.length; i++) {
      var axis = axes.elementAt(i);
      var data =
          _datasources[axis.name] = await dataSource.getMap(axis.dataType);
      var initialValue = data.length == 1 ? data.entries.first.key : null;
      _values[axis.name] = values?.elementAt(i) ?? initialValue;
    }
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
      future: _initialize(),
      builder: (context, snapshot) => Column(
        children: [
          for (var axis in axes)
            SelectFormField(
              decoration: InputDecoration(labelText: axis.title),
              type: SelectFormFieldType.dropdown,
              initialValue: _values[axis.name],
              items: _items(axis.name),
              labelText: axis.title,
              onChanged: (value) => _change(axis.name, value),
            )
        ],
      ),
    );
  }

  List<Map<String, dynamic>> _items(String name) {
    var items = _datasources[name]!
        .entries
        .map((e) => {'value': e.key, 'label': e.value})
        .toList();
    items.add({'value': null, 'label': 'none'});
    return items;
  }

  _change(String axis, String? value) async {
    _values[axis] = value;
    try {
      final constraints = await cube.constraints(cubeInfo.name, axes: _values);
      setState(() {
        for (var element in constraints.keys) {
          _datasources[element] = constraints[element]!;
        }
      });
    } catch (e) {
      print(e);
    }
    widget.onChanged(_values);
  }
}

==> 0.0.4.tar.gz: lib/controls/combination_pool.dart <==
library bizdoc;

import 'package:flutter/material.dart';
import 'package:select_form_field/select_form_field.dart';

import 'package:bizdoc/services/cube.dart';
import 'package:bizdoc/services/datasource.dart';
import 'package:bizdoc/services/session.dart';
import 'package:bizdoc/models/profile.dart';

typedef AxesArgs = void Function(Map<String, dynamic> axes);

class CombinationPool extends StatefulWidget {
  final String? labelText;
  final String? initialValue;
  final AxesArgs onChanged;
  final String? cube;
  CombinationPool(
      {this.cube, required this.onChanged, this.labelText, this.initialValue});
  @override
  _CombinationPoolState createState() => _CombinationPoolState();
}

class _CombinationPoolState extends State<CombinationPool> {
  final Map<String, dynamic> _values = {};
  final Map<String, Map<String, dynamic>> _datasources = {};

  late CubeInfo cubeInfo;
  late Iterable<CubeAxis> axes;

  @override
  void initState() {
    cubeInfo = session.profile.cubes.firstWhere(
        (element) => widget.cube == null || element.name == widget.cube);
    axes = cubeInfo.axes.where((element) => element.combination);
    super.initState();
  }

  _initialize() async {
    final values =
        widget.initialValue != null ? widget.initialValue!.split('.') : null;
    for (var i = 0; i < axes.length; i++) {
      var axis = axes.elementAt(i);
      var data = _datasources[axis.name] = await dataSource.map(axis.dataType);
      var initialValue = data.length == 1 ? data.entries.first.key : null;
      _values[axis.name] = values?.elementAt(i) ?? initialValue;
    }
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
      future: _initialize(),
      builder: (context, snapshot) => Column(
        children: [
          for (var axis in axes)
            SelectFormField(
              decoration: InputDecoration(labelText: axis.title),
              type: SelectFormFieldType.dropdown,
              initialValue: _values[axis.name],
              items: _items(axis.name),
              labelText: axis.title,
              onChanged: (value) => _change(axis.name, value),
            )
        ],
      ),
    );
  }

  List<Map<String, dynamic>> _items(String name) {
    var items = _datasources[name]!
        .entries
        .map((e) => {'value': e.key, 'label': e.value})
        .toList();
    items.add({'value': null, 'label': 'none'});
    return items;
  }

  _change(String axis, String? value) async {
    _values[axis] = value;
    try {
      final constraints = await cube.constraints(cubeInfo.name, axes: _values);
      setState(() {
        for (var element in constraints.keys) {
          _datasources[element] = constraints[element]!;
        }
      });
    } catch (e) {
      print(e);
    }
    widget.onChanged(_values);
  }
}
